/*!
 * rivet-core - @version 2.0.0-alpha.7

 * Copyright (C) 2018 The Trustees of Indiana University
 * SPDX-License-Identifier: BSD-3-Clause
 */

if (!Element.prototype.closest) {
  if (!Element.prototype.matches) {
    Element.prototype.matches =
      Element.prototype.msMatchesSelector ||
      Element.prototype.webkitMatchesSelector;
  }
  Element.prototype.closest = function(s) {
    var el = this;
    var ancestor = this;
    if (!document.documentElement.contains(el)) return null;
    do {
      if (ancestor.matches(s)) return ancestor;
      ancestor = ancestor.parentElement;
    } while (ancestor !== null);
    return null;
  };
}


(function() {
  if (typeof window.CustomEvent === 'function') return false;

  function CustomEvent(event, params) {
    params = params || { bubbles: false, cancelable: false, detail: undefined };
    var evt = document.createEvent('CustomEvent');
    evt.initCustomEvent(
      event,
      params.bubbles,
      params.cancelable,
      params.detail
    );
    return evt;
  }

  CustomEvent.prototype = window.Event.prototype;

  window.CustomEvent = CustomEvent;
})();

(function(arr) {
  arr.forEach(function(item) {
    if (item.hasOwnProperty('remove')) {
      return;
    }

    Object.defineProperty(item, 'remove', {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function remove() {
        if (this.parentNode === null) {
          return;
        }
        this.parentNode.removeChild(this);
      }
    });
  });
})([Element.prototype, CharacterData.prototype, DocumentType.prototype]);

const globalSettings = {
  prefix: 'rvt'
};

var Lie = typeof Promise === 'function' ? Promise : function (fn) {
  let queue = [], resolved = 0, value;
  fn($ => {
    value = $;
    resolved = 1;
    queue.splice(0).forEach(then);
  });
  return {then};
  function then(fn) {
    return (resolved ? setTimeout(fn, 0, value) : queue.push(fn)), this;
  }
};

const {document: document$1, MutationObserver: MutationObserver$1, Set: Set$1, WeakMap: WeakMap$1} = self;

const elements = element => 'querySelectorAll' in element;
const {filter} = [];

var QSAO = options => {
  const live = new WeakMap$1;
  const callback = records => {
    const {query} = options;
    if (query.length) {
      for (let i = 0, {length} = records; i < length; i++) {
        loop(filter.call(records[i].addedNodes, elements), true, query);
        loop(filter.call(records[i].removedNodes, elements), false, query);
      }
    }
  };
  const drop = elements => {
    for (let i = 0, {length} = elements; i < length; i++)
      live.delete(elements[i]);
  };
  const flush = () => {
    callback(observer.takeRecords());
  };
  const loop = (elements, connected, query, set = new Set$1) => {
    for (let selectors, element, i = 0, {length} = elements; i < length; i++) {
      if (!set.has(element = elements[i])) {
        set.add(element);
        if (connected) {
          for (let q, m = matches(element), i = 0, {length} = query; i < length; i++) {
            if (m.call(element, q = query[i])) {
              if (!live.has(element))
                live.set(element, new Set$1);
              selectors = live.get(element);
              if (!selectors.has(q)) {
                selectors.add(q);
                options.handle(element, connected, q);
              }
            }
          }
        }
        else if (live.has(element)) {
          selectors = live.get(element);
          live.delete(element);
          selectors.forEach(q => {
            options.handle(element, connected, q);
          });
        }
        loop(querySelectorAll(element), connected, query, set);
      }
    }
  };
  const matches = element => (
    element.matches ||
    element.webkitMatchesSelector ||
    element.msMatchesSelector
  );
  const parse = (elements, connected = true) => {
    loop(elements, connected, options.query);
  };
  const querySelectorAll = root => query.length ?
                            root.querySelectorAll(query) : query;
  const observer = new MutationObserver$1(callback);
  const root = options.root || document$1;
  const {query} = options;
  observer.observe(root, {childList: true, subtree: true});
  parse(querySelectorAll(root));
  return {drop, flush, observer, parse};
};

const {create, keys} = Object;

const attributes = new WeakMap;
const lazy = new Set;

const query = [];
const config = {};
const defined = {};

const attributeChangedCallback = (records, o) => {
  for (let h = attributes.get(o), i = 0, {length} = records; i < length; i++) {
    const {target, attributeName, oldValue} = records[i];
    const newValue = target.getAttribute(attributeName);
    h.attributeChanged(attributeName, oldValue, newValue);
  }
};

const set = (value, m, l, o) => {
  const handler = create(o, {element: {enumerable: true, value}});
  for (let i = 0, {length} = l; i < length; i++)
    value.addEventListener(l[i].t, handler, l[i].o);
  m.set(value, handler);
  if (handler.init)
    handler.init();
  const {observedAttributes} = o;
  if (observedAttributes) {
    const mo = new MutationObserver(attributeChangedCallback);
    mo.observe(value, {
      attributes: true,
      attributeOldValue: true,
      attributeFilter: observedAttributes.map(attributeName => {
        if (value.hasAttribute(attributeName))
          handler.attributeChanged(
            attributeName,
            null,
            value.getAttribute(attributeName)
          );
        return attributeName;
      })
    });
    attributes.set(mo, handler);
  }
  return handler;
};

const {drop, flush, parse} = QSAO({
  query,
  handle(element, connected, selector) {
    const {m, l, o} = config[selector];
    const handler = m.get(element) || set(element, m, l, o);
    const method = connected ? 'connected' : 'disconnected';
    if (method in handler)
      handler[method]();
  }
});

const define = (selector, definition) => {
  if (-1 < query.indexOf(selector))
    throw new Error('duplicated: ' + selector);
  flush();
  const listeners = [];
  const retype = create(null);
  for (let k = keys(definition), i = 0, {length} = k; i < length; i++) {
    const key = k[i];
    if (/^on/.test(key) && !/Options$/.test(key)) {
      const options = definition[key + 'Options'] || false;
      const lower = key.toLowerCase();
      let type = lower.slice(2);
      listeners.push({t: type, o: options});
      retype[type] = key;
      if (lower !== key) {
        type = key.slice(2, 3).toLowerCase() + key.slice(3);
        retype[type] = key;
        listeners.push({t: type, o: options});
      }
    }
  }
  if (listeners.length) {
    definition.handleEvent = function (event) {
      this[retype[event.type]](event);
    };
  }
  query.push(selector);
  config[selector] = {m: new WeakMap, l: listeners, o: definition};
  parse(document.querySelectorAll(selector));
  whenDefined(selector);
  if (!lazy.has(selector))
    defined[selector]._();
};

const whenDefined = selector => {
  if (!(selector in defined)) {
    let _, $ = new Lie($ => { _ = $; });
    defined[selector] = {_, $};
  }
  return defined[selector].$;
};

class Component {
  static initAll() {
    this.init(this.selector);
  }

  static init(selector) {
    define(selector, this.methods);

    return document.querySelector(selector);
  }

  static get selector() {
  }

  static get methods() {
  }

  static bindMethodToDOMElement(self, name, method) {
    Object.defineProperty(self.element, name, {
      value: method.bind(self),
      writable: false
    });
  }

  static dispatchCustomEvent(eventName, element, detail = {}) {
    const prefix = globalSettings.prefix;
    const event = new CustomEvent(`${prefix}:${eventName}`, {
      bubbles: true,
      cancelable: true,
      detail
    });

    return element.dispatchEvent(event);
  }

  static dispatchComponentAddedEvent(element) {
    return this.dispatchCustomEvent('componentAdded', document, {
      component: element
    });
  }

  static dispatchComponentRemovedEvent(element) {
    return this.dispatchCustomEvent('componentRemoved', document, {
      component: element
    });
  }
}


const nodeListToArray = nodes => Array.prototype.slice.call(nodes);


const keyCodes = {
  up: 38,
  down: 40,
  left: 37,
  right: 39,
  tab: 9,
  enter: 13,
  escape: 27,
  home: 36,
  end: 35,
  pageUp: 33,
  pageDown: 34
};


class Accordion extends Component {
  static get selector() {
    return '[data-rvt-accordion]';
  }

  static get methods() {
    return {
      init() {
        console.log('Accordion::init()');

        this.openAllOnInit = this.element.hasAttribute('data-rvt-accordion-open-all');
        this.triggerAttribute = 'data-rvt-accordion-trigger';
        this.triggerSelector = `[${this.triggerAttribute}]`;
        this.triggers = nodeListToArray(
          this.element.querySelectorAll(this.triggerSelector)
        );

                if (this.triggers.length < 2) {
          console.warn(
            'An accordion should contain *at least two* accordion triggers with the "data-rvt-accordion-trigger" attribute'
          );
        }

                this.panelAttribute = 'data-rvt-accordion-panel';
        this.panelSelector = `[${this.panelAttribute}]`;
        this.panels = nodeListToArray(
          this.element.querySelectorAll(this.panelSelector)
        );

                const initializedPanels = this.element.querySelectorAll(
          `[data-rvt-accordion-panel-init]`
        );

        try {
          if (initializedPanels.length > 1) {
            console.warn('Caught');
          }

              let initialPanel;

          this.panels.forEach((panel, index) => {
            if (panel.hasAttribute('data-rvt-accordion-panel-init')) {
              initialPanel = panel;
            } else {
              this.panels[index].setAttribute('hidden', '');
            }
          });

          this.openOnInit = initialPanel.getAttribute(this.panelAttribute);
        } catch (e) {
          console.warn(
            'Only one accordion panel should have the data-rvt-accordion-panel-init attribute. If you wish to open all panels on initialization, please apply the appropriate attribute to the data-rvt-accordion element'
          );
        }

        Component.bindMethodToDOMElement(this, 'open', this.open);
        Component.bindMethodToDOMElement(this, 'close', this.close);

        this._handleClick = this._handleClick.bind(this);
        this._handleKeydown = this._handleKeydown.bind(this);
      },

      connected() {
        Component.dispatchComponentAddedEvent(this.element);

        this._openOnInit();
      },

      disconnected() {
        Component.dispatchComponentRemovedEvent(this.element);
      },

      onClick(event) {
        this._handleClick(event);
      },

      onKeydown(event) {
        this._handleKeydown(event);
      },

      _openOnInit() {
        if (this.openAllOnInit === true) {
          this.panels.forEach(panel => {
            this.open(panel.getAttribute(this.panelAttribute));
          });
        } else {
          this.panels.forEach((panel, index) => {
            if (panel.getAttribute(this.panelAttribute) !== this.openOnInit) {
              this.panels[index].setAttribute('hidden', '');
              const trigger = this.element.querySelector(
                `[${this.triggerAttribute}="${panel.dataset.rvtAccordionPanel}"]`
              );
              trigger.setAttribute('aria-expanded', 'false');
            } else {
              this.open(this.openOnInit);
            }
          });
        }
      },

      _handleClick(event) {
        const currentTrigger = event.target.closest(this.triggerSelector);
        if (!currentTrigger) return;

        const currentTriggerValue = currentTrigger.getAttribute(
          this.triggerAttribute
        );

        currentTrigger.getAttribute('aria-expanded') === 'true'
          ? this.close(currentTriggerValue)
          : this.open(currentTriggerValue);
      },

      _handleKeydown(event) {
        if (
          event.keyCode === keyCodes.up ||
          event.keyCode === keyCodes.down ||
          event.keyCode === keyCodes.end ||
          event.keyCode === keyCodes.home
        ) {
          const accordionParent = event.target.closest('[data-rvt-accordion]');
          if (!accordionParent) return;

              const currentTrigger = event.target.closest(this.triggerSelector);
          if (!currentTrigger) return;

          const nextTrigger = this.triggers.indexOf(currentTrigger) + 1;
          const prevTrigger = this.triggers.indexOf(currentTrigger) - 1;

              switch (event.keyCode) {
            case keyCodes.up: {
              if (this.triggers[prevTrigger] === undefined) {
                this.triggers[this.triggers.length - 1].focus();
              } else {
                this.triggers[prevTrigger].focus();
              }
              break;
            }

                case keyCodes.down: {
              if (this.triggers[nextTrigger] === undefined) {
                this.triggers[0].focus();
              } else {
                this.triggers[nextTrigger].focus();
              }
              break;
            }

                case keyCodes.end: {
              this.triggers[this.triggers.length - 1].focus();
              break;
            }

                case keyCodes.home: {
              this.triggers[0].focus();
              break;
            }
          }
        }
      },

      open(panelId) {
        const panel = this.element.querySelector(
          `[${this.panelAttribute}="${panelId}"]`
        );

        if (!panel) {
          console.warn(`No such panel '${panelId}' in Accordion.open()`);
          return;
        }

            const activationEvent = Component.dispatchCustomEvent(
          'accordionOpened',
          this.element,
          { panel }
        );

            if (!activationEvent) return;

            const trigger = this.element.querySelector(
          `[${this.triggerAttribute} = "${panel.dataset.rvtAccordionPanel}"]`
        );

        console.log(`[${this.triggerAttribute} = "${panel.dataset.rvtAccordionPanel}"]`);

        panel.removeAttribute('hidden');
        trigger.setAttribute('aria-expanded', 'true');
      },

      close(panelId) {
        const panel = this.element.querySelector(
          `[${this.panelAttribute}="${panelId}"]`
        );

        if (!panel) {
          console.warn(`No such panel '${panelId}' in Accordion.close()`);
          return;
        }

            const activationEvent = Component.dispatchCustomEvent(
          'accordionClosed',
          this.element,
          { panel }
        );

            if (!activationEvent) return;

            const trigger = this.element.querySelector(
          `[${this.triggerAttribute} = "${panel.dataset.rvtAccordionPanel}"]`
        );

        panel.setAttribute('hidden', '');
        trigger.setAttribute('aria-expanded', 'false');
      }
    }
  }
}


class Alert extends Component {
  static get selector() {
    return '[data-rvt-alert]';
  }

  static get methods() {
    return {
      init() {
        console.log('Alert::init()');

        this.closeAttribute = 'data-rvt-alert-close';
        this.closeSelector = `[${this.closeAttribute}]`;
        this.closeButton = null;

        Component.bindMethodToDOMElement(this, 'dismiss', this.dismiss);

                this._handleClick = this._handleClick.bind(this);
      },

      connected() {
        Component.dispatchComponentAddedEvent(this.element);

        this.closeButton = this.element.querySelector(this.closeSelector);
        this.closeButton.addEventListener('click', this._handleClick, false);
      },

      disconnected() {
        Component.dispatchComponentRemovedEvent(this.element);

                this.closeButton.removeEventListener('click', this._handleClick, false);
      },

      _handleClick() {
        this.dismiss();
      },

          dismiss() {
        const dismissEvent = Component.dispatchCustomEvent(
          'alertDismissed',
          this.element
        );

            if (!dismissEvent) return;

            this.element.remove();
      }
    }
  }
}

class Disclosure extends Component {
  static get selector() {
    return '[data-rvt-disclosure]';
  }

  static get methods() {
    return {
      init() {
        console.log('Disclosure::init');

        this.toggleElement = this.element.querySelector('[data-rvt-disclosure-toggle]');
        this.targetElement = this.element.querySelector('[data-rvt-disclosure-target]');
        this.isOpen = false;
        this.activeToggle = null;
        this.activeDisclosure = null;

                const icon = this.element.querySelector('svg');
        if (icon) {
          icon.setAttribute('focusable', 'false');
        }

                Component.bindMethodToDOMElement(this, 'open', this.open);
        Component.bindMethodToDOMElement(this, 'close', this.close);

        this._handleClick = this._handleClick.bind(this);
        this._handleKeydown = this._handleKeydown.bind(this);
      },

      connected() {
        Component.dispatchComponentAddedEvent(this.element);

                document.addEventListener('click', this._handleClick, false);
        document.addEventListener('keydown', this._handleKeydown, false);
      },

      disconnected() {
        Component.dispatchComponentRemovedEvent(this.element);

        document.removeEventListener('click', this._handleClick, false);
        document.removeEventListener('keydown', this._handleKeydown, false);
      },

      open() {

        if (this.toggleElement.hasAttribute('disabled')) {
          return;
        }


        const openEvent = Component.dispatchCustomEvent(
          'disclosureOpened',
          this.element
        );


        if (!openEvent) return;


        this.isOpen = true;
        this.toggleElement.setAttribute('aria-expanded', 'true');


        this.targetElement.removeAttribute('hidden');


        this.activeToggle = this.toggleElement;
        this.activeDisclosure = this.targetElement;
      },

      close() {
        if (!this.activeToggle) return;

        const closeEvent = Component.dispatchCustomEvent(
          'disclosureClosed',
          this.element
        );

        if (!closeEvent) return;

        this.isOpen = false;
        this.activeToggle.setAttribute('aria-expanded', 'false');
        this.activeDisclosure.setAttribute('hidden', '');

        this.activeToggle = null;
        this.activeDisclosure = null;
      },

      _handleClick(event) {
        const toggle = event.target.closest('[data-rvt-disclosure-toggle]');

        if (this.targetElement.contains(event.target)) return;

        if (!toggle && this.activeToggle !== null) {
          this.close();
          return;
        }

        if (toggle !== this.toggleElement || this.isOpen) {
          this.close();
        } else {
          this.open();
        }
      },

      _shouldHandleKeydown(event) {
        if (!this.element.contains(event.target)) return false;

        const disclosure = event.target.closest('[data-rvt-disclosure]');
        if (disclosure !== this.element) return false;

            return true;
      },

          _handleKeydown(event) {
        if (!this._shouldHandleKeydown(event)) return;

            switch (event.keyCode) {
          case keyCodes.escape: {
            if (!this.activeToggle) return;

            this.close();

                this.toggleElement.focus();

            this.activeToggle = null;

                break;
          }
        }
      }
    };
  }
}



class Dropdown extends Component {
  static get selector() {
    return '[data-rvt-dropdown]';
  }

  static get methods() {
    return {
      init() {
        console.log('Dropdown::init()');

        this.disclosureAttribute = '[data-rvt-dropdown]';
        this.toggleAttribute = '[data-rvt-dropdown-toggle]';
        this.toggleDataProperty = 'dropdownToggle';
        this.targetAttribute = '[data-rvt-dropdown-menu]';
        this.openEventName = 'dropdownOpen';
        this.closeEventNane = 'dropdownClose';
        this.toggleElement = this.element.querySelector('[data-rvt-dropdown-toggle]');
        this.targetElement = this.element.querySelector('[data-rvt-dropdown-menu]');
        this.isOpen = false;
        this.activeToggle = null;
        this.activeDisclosure = null;

                const icon = this.element.querySelector('svg');
        if (icon) {
          icon.setAttribute('focusable', 'false');
        }

                Component.bindMethodToDOMElement(this, 'open', this.open);
        Component.bindMethodToDOMElement(this, 'close', this.close);

        this._handleClick = this._handleClick.bind(this);
        this._handleKeydown = this._handleKeydown.bind(this);

        this.focusableElements = 'a[href], area[href], input:not([disabled]), select:not([disabled]), textarea:not([disabled]), button:not([disabled]), [tabindex="0"]';
      },

      connected() {
        Component.dispatchComponentAddedEvent(this.element);

                document.addEventListener('click', this._handleClick, false);
        document.addEventListener('keydown', this._handleKeydown, false);
      },

      disconnected() {
        Component.dispatchComponentRemovedEvent(this.element);

        document.removeEventListener('click', this._handleClick, false);
        document.removeEventListener('keydown', this._handleKeydown, false);
      },

      open() {

        if (this.toggleElement.hasAttribute('disabled')) {
          return;
        }


        const openEvent = Component.dispatchCustomEvent(
          'dropdownOpened',
          this.element
        );


        if (!openEvent) return;


        this.isOpen = true;
        this.toggleElement.setAttribute('aria-expanded', 'true');


        this.targetElement.removeAttribute('hidden');


        this.activeToggle = this.toggleElement;
        this.activeDisclosure = this.targetElement;
      },

      close() {
        if (!this.activeToggle) return;

        const closeEvent = Component.dispatchCustomEvent(
          'dropdownClosed',
          this.element
        );

        if (!closeEvent) return;

        this.isOpen = false;
        this.activeToggle.setAttribute('aria-expanded', 'false');
        this.activeDisclosure.setAttribute('hidden', '');

        this.activeToggle = null;
        this.activeDisclosure = null;
      },

      _setUpMenu(menu) {
        const menuObject = {};

        const menuFocusables = nodeListToArray(
          menu.querySelectorAll(this.focusableElements)
        );

        menuObject.all = menuFocusables;

        menuObject.first = menuFocusables[0];

        menuObject.last = menuFocusables[menuFocusables.length - 1];

            return menuObject;
      },

      _handleClick(event) {
        const toggle = event.target.closest('[data-rvt-dropdown-toggle]');

        if (this.targetElement.contains(event.target)) return;

        if (!toggle && this.activeToggle !== null) {
          this.close();
          return;
        }

        if (toggle !== this.toggleElement || this.isOpen) {
          this.close();
        } else {
          this.open();
        }
      },

      _shouldHandleKeydown(event) {
        if (!this.element.contains(event.target)) return false;

        const disclosure = event.target.closest('[data-rvt-dropdown]');
        if (disclosure !== this.element) return false;

            return true;
      },

      _handleKeydown(event) {
        if (!this._shouldHandleKeydown(event)) return;

            switch (event.keyCode) {
          case keyCodes.escape: {
            if (!this.activeToggle) return;

            this.close();

                this.toggleElement.focus();

            this.activeToggle = null;

                break;
          }

          case keyCodes.down: {
            if (!this.isOpen) {
              this.open();
            } else {
              const currentMenu = this._setUpMenu(this.targetElement);
              currentMenu.first.focus();
            }

            if (!this.targetElement.contains(event.target)) break;

            const currentMenu = this._setUpMenu(this.targetElement);
            let currentIndex;

                for (let i = 0; i < currentMenu.all.length; i++) {
              if (event.target == currentMenu.all[i]) {
                currentIndex = i;
              }
            }

                const nextItem = currentMenu.all[currentIndex + 1];

                if (!nextItem) {
              currentMenu.first.focus();

                  return;
            }

                nextItem.focus();

                break;
          }

              case keyCodes.up: {
            event.preventDefault();

            if (!this.targetElement.contains(event.target)) break;

            const currentMenu = this._setUpMenu(this.targetElement);
            let currentIndex;

                for (let i = 0; i < currentMenu.all.length; i++) {
              if (event.target == currentMenu.all[i]) {
                currentIndex = i;
              }
            }

                const previousItem = currentMenu.all[currentIndex - 1];

                if (!previousItem && currentMenu.last !== undefined) {
              currentMenu.last.focus();

                  return;
            }

                previousItem.focus();

                break;
          }

              case keyCodes.tab: {
            if (!this.targetElement.contains(event.target)) break;

                const currentMenu = this._setUpMenu(this.targetElement);

            if (document.activeElement == currentMenu.last && !event.shiftKey) {
              this.close();

                  return;
            }

                break;
          }
        }
      }
    }
  }
}


class FileInput extends Component {
  static get selector() {
    return '[data-rvt-file-input]';
  }

  static get methods() {
    return {
      init() {
        console.log('FileInput::init()');

        this.wrapperAttribute = 'data-rvt-file-input';
        this.inputAttribute = 'data-rvt-file-input-button';
        this.previewAttribute = 'data-rvt-file-input-preview';
        this.previewText = this.element.querySelector(`[${this.previewAttribute}]`).textContent;

            this._handleChange = this._handleChange.bind(this);
      },

      connected() {
        Component.dispatchComponentAddedEvent(this.element);
      },

      disconnected() {
        Component.dispatchComponentRemovedEvent(this.element);
      },

      _sanitizeHTML(str) {
        const temp = document.createElement('div');
        temp.textContent = str;
        return temp.innerHTML;
      },

      _buildSingleFile(input) {
        const singleFileItem = document.createElement('span');

        const singleFileName = this._sanitizeHTML(input.files[0].name);

        singleFileItem.textContent = singleFileName;

        return singleFileItem;
      },

      _buildMultipleFiles(input) {
        const fileCount = document.createElement('span');
        fileCount.textContent = input.files.length + ' files selected';

            return fileCount;
      },

      onChange(event) {
        this._handleChange();
      },

      _handleChange() {
        const uploadInput = this.element.querySelector(`[${this.inputAttribute}]`);

        const uploadPreview = this.element.querySelector(
          `[${this.previewAttribute}]`
        );

        if (uploadInput.files.length > 0) {
          const attachedEvent = Component.dispatchCustomEvent(
            'fileAttached',
            this.element,
            {
              files: Array.from(uploadInput.files).map(f => f.name)
            }
          );

          if (!attachedEvent) return;

          uploadPreview.innerHTML = '';

          uploadInput.files.length > 1
            ? uploadPreview.appendChild(this._buildMultipleFiles(uploadInput))
            : uploadPreview.appendChild(this._buildSingleFile(uploadInput));
        } else {
          uploadPreview.innerHTML = this.previewText;
        }
      }
    }
  }
}


class Modal extends Component {
  static get selector() {
    return '[data-rvt-modal]';
  }

  static get methods() {
    return {
      init() {
        console.log('Modal::init()');

        this.openOnInit = this.element.hasAttribute('data-rvt-modal-open');
        this.dialog = this.element.hasAttribute('data-rvt-modal-dialog');
        this.modalAttribute = 'data-rvt-modal';
        this.modalSelector = `[${this.modalAttribute}]`;
        this.modalDataValue = this.element.getAttribute('data-rvt-modal');
        this.innerModalAttribute = 'data-rvt-modal-inner';
        this.innerModalSelector = `[${this.innerModalAttribute}]`;
        this.openAttribute = 'data-rvt-modal-trigger';
        this.openSelector = `[${this.openAttribute}]`;
        this.openButton = document.querySelector(
          `[${this.openAttribute}="${this.modalDataValue}"]`
        );
        this.closeAttribute = 'data-rvt-modal-close';
        this.closeSelector = `[${this.closeAttribute}]`;

        this.focusElements =
          'a[href], area[href], input:not([disabled]), select:not([disabled]), textarea:not([disabled]), button:not([disabled]), [tabindex="-1"]';

        Component.bindMethodToDOMElement(this, 'open', this.open);
        Component.bindMethodToDOMElement(this, 'close', this.close);
        Component.bindMethodToDOMElement(this, 'focusTrigger', this.focusTrigger);
        Component.bindMethodToDOMElement(this, 'focusModal', this.focusModal);

        this._handleClick = this._handleClick.bind(this);
        this._handleKeydown = this._handleKeydown.bind(this);
      },

      connected() {
        Component.dispatchComponentAddedEvent(this.element);

        if (!this.openOnInit) {
          this.element.setAttribute('hidden', '');
        }

        document.addEventListener('click', this._handleClick, false);
        document.addEventListener('keydown', this._handleKeydown, false);
      },

      disconnected() {
        Component.dispatchComponentRemovedEvent(this.element);

                document.removeEventListener('click', this._handleClick, false);
        document.removeEventListener('keydown', this._handleKeydown, false);
      },

      _handleClick(event) {
        event.target.closest(this.innerModalSelector) !== null
          ? (event.clickedInModal = true)
          : (event.clickedInModal = false);

            if (event.clickedInModal) {
          event.stopPropagation();
        }

        const triggerSelectors = `${this.openSelector}, ${this.closeSelector}, ${this.modalSelector}`;
        const trigger = event.target.closest(triggerSelectors);
        if (!trigger) return;

        let triggerContent;

            if (trigger.getAttribute(this.openAttribute)) {
          triggerContent = trigger.getAttribute(this.openAttribute);
        } else if (trigger.getAttribute(this.closeAttribute)) {
          triggerContent = trigger.getAttribute(this.closeAttribute);
        } else {
          triggerContent = event.target.closest(this.modalSelector);
        }

        if (!triggerContent) return;

            switch (trigger != null) {
          case trigger.hasAttribute(this.openAttribute): {
            if (trigger.getAttribute(this.openAttribute) !== this.modalDataValue) {
              this.close();
              return;
            }

                this.open();

                this.element.focus();

                break;
          }
          case trigger.hasAttribute(this.closeAttribute): {
            event.preventDefault();

            if (trigger.getAttribute(this.closeAttribute) !== this.modalDataValue)
              return;
            this.close();

                if (this.openButton !== null) this.openButton.focus();

                break;
          }
          case !event.clickedInModal: {
            if (this.dialog) return;

            if (
              triggerContent.getAttribute(this.modalAttribute) !==
              this.modalDataValue
            )
              return;
            this.close();

                if (this.openButton !== null) this.openButton.focus();

                break;
          }
          default: {
            return;
          }
        }
      },

      _handleBackwardTab(first, last, event) {
        if (
          document.activeElement === first ||
          document.activeElement === this.element
        ) {
          event.preventDefault();
          last.focus();
        }
      },

      _handleForwardTab(first, last, event) {
        if (document.activeElement === last) {
          event.preventDefault();
          first.focus();
        }
      },

      _handleKeydown(event) {
        const currentModal = event.target.closest(this.modalSelector);
        if (!currentModal) return;

            switch (event.keyCode) {
          case keyCodes.tab: {
            const focusList = nodeListToArray(
              currentModal.querySelectorAll(this.focusElements)
            );
            const firstFocus = focusList[0];
            const finalFocus = focusList[focusList.length - 1];

            event.shiftKey
              ? this._handleBackwardTab(firstFocus, finalFocus, event)
              : this._handleForwardTab(firstFocus, finalFocus, event);

                break;
          }
          case keyCodes.escape: {
            if (this.dialog) return;

            if (
              currentModal.getAttribute(this.modalAttribute) !== this.modalDataValue
            )
              return;

                this.close();

                if (this.openButton !== null) this.openButton.focus();

                break;
          }
        }
      },

      open() {
        const openEvent = Component.dispatchCustomEvent(
          'modalOpened',
          this.element
        );

            if (!openEvent) return;

            this.element.removeAttribute('hidden');
        document.body.classList.add('rvt-modal-open');
      },

      close() {
        const closeEvent = Component.dispatchCustomEvent(
          'modalClosed',
          this.element
        );

            if (!closeEvent) return;

            this.element.setAttribute('hidden', '');
        document.body.classList.remove('rvt-modal-open');
      },

          focusTrigger() {
        if (!this.openButton) {
          throw new Error(
            `Could not find a modal trigger with the value of ${this.modalDataValue}`
          );
        }

            this.openButton.focus();
      },

          focusModal() {
        if (!this.element) {
          throw new Error(
            `Could not find a modal with the value of ${this.modalDataValue}`
          );
        }

            this.element.focus();
      }

    }
  }
}


class Sidenav extends Component {
  static get selector() {
    return '[data-rvt-sidenav]';
  }

  static get methods() {
    return {
      init() {
        console.log('Sidenav::init');

        this.toggleAttribute = 'data-rvt-sidenav-toggle';
        this.toggleSelector = `[${this.toggleAttribute}]`;
        this.listAttribute = 'data-rvt-sidenav-list';
        this.listSelector = `[${this.listAttribute}]`;
        this.openAllOnInit = this.element.hasAttribute('data-rvt-sidenav-open-all');

        const menuToggles = nodeListToArray(
          this.element.querySelectorAll(this.toggleSelector)
        );
        const childMenus = nodeListToArray(
          this.element.querySelectorAll(this.listSelector)
        );

            menuToggles.forEach((element, index) => {

          element.setAttribute('aria-haspopup', 'true');

          if (this.openAllOnInit) {
            element.setAttribute('aria-expanded', 'true');
            childMenus[index].removeAttribute('hidden');

                return;
          }

          if (element.getAttribute('aria-expanded') === 'true') {
            childMenus[index].removeAttribute('hidden');
          } else {
            element.setAttribute('aria-expanded', 'false');
            childMenus[index].setAttribute('hidden', '');
          }
        });

        Component.bindMethodToDOMElement(this, 'open', this.open);
        Component.bindMethodToDOMElement(this, 'close', this.close);

        this._handleClick = this._handleClick.bind(this);
      },

      connected() {
        Component.dispatchComponentAddedEvent(this.element);
      },

      disconnected() {
        Component.dispatchComponentRemovedEvent(this.element);
      },

      onClick(event) {
        this._handleClick(event);
      },

      _handleClick(event) {
        const toggleButton = event.target.closest(this.toggleSelector);
        if (!toggleButton) return;

            const toggleId = toggleButton.dataset.rvtSidenavToggle;
        const targetList = this.element.querySelector(
          `[${this.listAttribute}="${toggleId}"]`
        );

        if (!targetList || targetList.getAttribute(this.listAttribute) === '') {
          return;
        }

            targetList.hasAttribute('hidden')
          ? this.open(toggleId)
          : this.close(toggleId);
      },

      open(toggleId) {
        const toggle = this.element.querySelector(
          `[${this.toggleAttribute}="${toggleId}"]`
        );
        const targetList = this.element.querySelector(
          `[${this.listAttribute}="${toggleId}"]`
        );

        if (!toggle || !targetList) {
          console.warn(`No such toggle '${toggleId}' in Sidenav.open()`);
          return;
        }

        const openEvent = Component.dispatchCustomEvent(
          'sidenavListOpened', 
          this.element,
          {
            list: targetList
          }
        );

            if (!openEvent) return;

            toggle.setAttribute('aria-expanded', 'true');
        targetList.removeAttribute('hidden');
      },

          close(toggleId) {
        const toggle = this.element.querySelector(
          `[${this.toggleAttribute}="${toggleId}"]`
        );
        const targetList = this.element.querySelector(
          `[${this.listAttribute}="${toggleId}"]`
        );
        if (!toggle || !targetList) {
          console.warn(`No such toggle '${toggleId}' in Sidenav.close()`);
          return;
        }

        const closeEvent = Component.dispatchCustomEvent(
          'sidenavListClosed', 
          this.element, 
          {
            list: targetList
          }
        );

            if (!closeEvent) return;

            toggle.setAttribute('aria-expanded', 'false');
        targetList.setAttribute('hidden', '');
      }
    }
  }
}


class Tabs extends Component {
  static get selector() {
    return '[data-rvt-tabs]';
  }

  static get methods() {
    return {
      init() {
        console.log('Tabs::init()');

                this.tablist = this.element.querySelector('[role="tablist"]');
        this.tabAttribute = 'data-rvt-tab';
        this.tabSelector = `[${this.tabAttribute}]`;
        this.tabs = nodeListToArray(
          this.element.querySelectorAll(this.tabSelector)
        );
        this.panelAttribute = 'data-rvt-tab-panel';
        this.panelSelector = `[${this.panelAttribute}]`;
        this.panels = nodeListToArray(
          this.element.querySelectorAll(this.panelSelector)
        );

        let initialPanel;
        this.panels.forEach((panel, index) => {
          if (panel.hasAttribute('data-rvt-tab-init')) {
            initialPanel = panel;
          } else {
            this.tabs[index].setAttribute('tabindex', '-1');
          }
        });

        this.openOnInit = initialPanel.getAttribute(this.panelAttribute) || this.panels[0].getAttribute(this.panelAttribute);

        console.log(this.openOnInit);

        Component.bindMethodToDOMElement(this, 'activateTab', this.activateTab);

        this._handleClick = this._handleClick.bind(this);
        this._handleKeydown = this._handleKeydown.bind(this);
      },

      connected() {
        Component.dispatchComponentAddedEvent(this.element);

        this.activateTab(this.openOnInit);

                this.tablist.addEventListener('click', this._handleClick, false);
        this.tablist.addEventListener('keydown', this._handleKeydown, false);
      },

      disconnected() {
        Component.dispatchComponentRemovedEvent(this.element);

        this.tablist.removeEventListener('click', this._handleClick, false);
        this.tablist.removeEventListener('keydown', this._handleKeydown, false);
      },

      _handleClick(event) {
        const currentTab = event.target.closest(this.tabSelector);
        if (!currentTab) return;

        const tabId = currentTab.getAttribute(this.tabAttribute);

        this.activateTab(tabId);
      },

      _handleKeydown(event) {
        const currentTab = event.target.closest(this.tabSelector);
        if (!currentTab) return;

        const nextTab = this.tabs.indexOf(currentTab) + 1;
        const prevTab = this.tabs.indexOf(currentTab) - 1;

            switch (event.keyCode) {
          case keyCodes.right : {
            !this.tabs[nextTab] ? this.tabs[0].focus() : this.tabs[nextTab].focus();
            break;
          }

              case keyCodes.down: {
            !this.tabs[nextTab] ? this.tabs[0].focus() : this.tabs[nextTab].focus();
            break;
          }

              case keyCodes.left: {
            !this.tabs[prevTab]
              ? this.tabs[this.tabs.length - 1].focus()
              : this.tabs[prevTab].focus();
            break;
          }

              case keyCodes.up: {
            !this.tabs[prevTab]
              ? this.tabs[this.tabs.length - 1].focus()
              : this.tabs[prevTab].focus();
            break;
          }

              case keyCodes.end: {
            this.tabs[this.tabs.length - 1].focus();
            break;
          }

              case keyCodes.home: {
            this.tabs[0].focus();
            break;
          }
        }
      },

      activateTab(tabId) {
        const tab = this.element.querySelector(
          `[${this.panelAttribute}="${tabId}"]`
        );

        if (!tab) {
          console.warn(`No such tab '${tabId}' in Tabs.activateTab()`);
          return;
        }

        const activationEvent = Component.dispatchCustomEvent(
          'tabActivated',
          this.element, 
          { tab }
        );

            if (!activationEvent) return;

            const trigger = this.element.querySelector(
          `[${this.tabAttribute}="${tab.dataset.rvtTabPanel}"]`
        );

            this.panels.forEach((panel, index) => {
          if (panel.getAttribute(this.panelAttribute) !== tab.dataset.rvtTabPanel) {
            panel.setAttribute('hidden', '');
            this.tabs[index].setAttribute('aria-selected', 'false');
            this.tabs[index].setAttribute('tabindex', '-1');
          }
        });

        tab.removeAttribute('hidden');
        trigger.setAttribute('aria-selected', 'true');
        trigger.removeAttribute('tabindex');
      }
    }
  }
}



function init() {
  console.log('Auto-initializing all Rivet components...');

  Accordion.initAll();
  Alert.initAll();
  Disclosure.initAll();
  Dropdown.initAll();
  FileInput.initAll();
  Modal.initAll();
  Sidenav.initAll();
  Tabs.initAll();
}

export { Accordion, Alert, Disclosure, Dropdown, FileInput, Modal, Sidenav, Tabs, init };
